import os
import tkinter as tk
from tkinter import ttk
from .exeptions import Warnings

class View:
    def __init__(self, cont):

        # Create a root window
        self.root = tk.Tk()

        # Assign the name of the window
        self.root.title('</POSTagger>')
        self.root.iconphoto(
            False, tk.PhotoImage(
                file=os.path.join(
                    os.path.dirname(os.path.abspath(__file__)),
                    'imgs',
                    'logo.png'
                )))

        # Set a theme to the window
        theme = ttk.Style()
        theme.theme_use('clam')

        # Set the window's size
        w = self.root.winfo_screenwidth()
        h = self.root.winfo_screenheight()
        self.root.geometry(f"{w}x{h}+0+0")

        # Configure the window layout
        self.root.columnconfigure(1, weight=2)

        # Make controller local
        self.cont = cont

        # Create error instance
        self.ER = Warnings()

        # Set padx, pady and sticky for all widgets
        self.options = {'padx': 5, 'pady': 5}

        self.textbox()
        self.wordfrequency()
        self.wordentry()
        self.editbuttons()
        self.categories()
        self.tagbuttons()
        self.menubar()

        # Update main parameters, textBox and wordEntry
        #self.cont.open_corpus()
        # self.update_textBox()
        # self.update_wordEntry()

        # self.root = root
        self.root.mainloop()

    def textbox(self):
        ############
        # Text Box #
        ############
        # Create a textBox fram and a textBox widget for sentences
        text_frame = ttk.Frame(self.root)
        self.sent_box = tk.Text(
            text_frame, 
            height=8, 
            width=60, 
            font=(
                self.cont.font_name, 
                self.cont.font))

        self.sent_box.tag_configure(
            self.cont.dir, 
            justify=self.cont.dir)

        self.sent_box.grid(
            row=0, 
            column=1, 
            ipady=5)
        
        # Create a scrollbar for textbox
        scroll = ttk.Scrollbar(
            text_frame, 
            orient='vertical', 
            command=self.sent_box.yview)

        scroll.grid(
            row=0, 
            column=2, 
            sticky='ns')

        # Set scrollbar to the textBox
        self.sent_box['yscrollcommand'] = scroll.set
        text_frame.grid(
            row=0, 
            column=1, 
            **self.options)
        self.update_textBox()
    
    def wordfrequency(self):
        ###################
        # Frequency Label #
        ###################
        # Get the frequencies
        c,l = self.cont.text.count(self.cont.word), self.cont.corpus_length
        # Create a label for frequency
        self.freq_label = ttk.LabelFrame(self.root)
        self.frequency = ttk.Label(
            self.freq_label,
            text=f'{c} occurrence in {l} words ({c/l if l else 0})'
            )
        self.frequency.grid()
        self.freq_label.grid(row=1, column=1)

    def wordentry(self):
        ##############
        # Word Entry #
        ##############
        # Create a word fram and a word entry widget for words
        word_frame = ttk.Frame(self.root)
        self.word_entry = ttk.Entry(
            word_frame, 
            width=50, 
            font = f'{self.cont.font_name} {self.cont.font} bold',
            justify='center')
        self.word_entry.grid(ipady=5)
        word_frame.grid(
            row=2, 
            column=1, 
            **self.options)
        self.update_wordEntry()

    def editbuttons(self):
        ################
        # Edit Buttons #
        ################
        # Create a merge, split, re-tokenize buttons
        ebutt_frame = ttk.Frame(self.root)
        split_btn = ttk.Button(
            ebutt_frame, 
            text='Re-Tokenize',
            command=self.retokenize)

        split_btn.grid(
            row=0, 
            column=0, 
            **self.options)

        # Split button
        edit_btn = ttk.Button(
            ebutt_frame, 
            text='Split',
            command=self.spliting)
        edit_btn.grid(
            row=0, 
            column=1, 
            **self.options)

        # Merge button
        comb_btn = ttk.Button(
            ebutt_frame, 
            text='Merge',
            command=self.merge)
        comb_btn.grid(
            row=0, 
            column=2, 
            **self.options)

        ebutt_frame.grid(row=3, column=1)

    def categories(self):
        ##############
        # Categories #
        ##############
        # Create tagset radioButtons
        tags = self.cont.get_tagset()
        self.cat_frame = ttk.LabelFrame(self.root)
        # POS radiobuttons
        self.tag = tk.StringVar()
        for idx, tup in enumerate(tags.items()):
            ttk.Radiobutton(
                self.cat_frame, 
                text=tup[0], 
                variable=self.tag, 
                value=tup[1]).grid(
                    row=idx//5, column=idx%5,
                    **self.options, 
                    sticky='w')
        self.cat_frame.grid(row=4, column=1)
        #return cat_frame

    def tagbuttons(self):
        ###################
        # Tagging Buttons #
        ###################
        # Create tagging, undo, and exit buttons
        tagging_button_frame = ttk.Frame(self.root)
        # Tag button
        next_btn = ttk.Button(
            tagging_button_frame, 
            text='Tag',
            command=self.tagging)

        next_btn.grid(
            row=0, 
            column=2, 
            **self.options)
        next_btn.focus_set()

        # Undo button
        back_btn = ttk.Button(
            tagging_button_frame, 
            text='Untag',
            command=self.untag_word)

        back_btn.grid(
            row=0, 
            column=1, 
            **self.options)

        tagging_button_frame.grid(
            row=5, 
            column=1, 
            **self.options)

    def menubar(self):
        ############
        # Menu Bar #
        ############
        # Create a menubar
        menubar = tk.Menu(self.root)
        self.root.config(menu=menubar)

        # Create a file, option, and help menu
        file_menu = tk.Menu(menubar, tearoff=0)
        help_menu = tk.Menu(menubar, tearoff=0)

        # Add the File menu to the menubar
        menubar.add_cascade(label="File", menu=file_menu)

        # Add menu items to the file menu
        file_menu.add_command(
            label='Open a Corpus',
            command=self.initiate)

        file_menu.add_command(
            label='Open a Project',
            command=lambda: self.initiate(False))

        file_menu.add_command(
            label='Open a language model', 
            state=tk.DISABLED)

        file_menu.add_command(
            label='Save the project',
            command=self.cont.save_project)
        file_menu.add_command(
            label='Export...',
            command=self.cont.export)
        file_menu.add_command(
            label='Preferences',
            command=self.option_window)
        file_menu.add_command(
            label='Exit', 
            command=self.exiting)

        # Add the option menu to the menubar
        menubar.add_cascade(label="Help", menu=help_menu)

        # Add menu items to the option menu
        help_menu.add_command(
            label='Tutorials', 
            command=self.root.destroy)

        help_menu.add_command(
            label='Documentation', 
            command=self.root.destroy)

    def initiate(self, corpus=True):
        if corpus:
            self.cont.open_corpus()
        else:
            self.cont.open_project()
        self.update_textBox()
        self.update_wordEntry()

    def update_textBox(self):
        # Redirect the the sentence
        sent = self.cont.redirect(' '.join(self.cont.sentence))
        # Update the textBox
        self.sent_box.delete('0.0', tk.END)
        self.sent_box.insert('0.0', sent, self.cont.dir)

    def update_wordEntry(self):
        # Redirect the the word
        word = self.cont.redirect(self.cont.word)
        # Update the frequency label
        c = self.cont.text.count(self.cont.word) if self.cont.word else 0
        l = self.cont.corpus_length
        p = c/l if l else 0
        self.frequency.config(
            text=f'{c} occurrence in {l} words ({p})')
        # Update the wordEntry
        self.word_entry.delete(0, tk.END)
        self.word_entry.insert(0, word)

    def update_categories(self):
        # Update tagset
        self.cat_frame.grid_forget()
        self.categories()

    def next_sentence(self):
        # Check if corpus is empty and the tagged corpus is full
        if not self.cont.corpus and self.cont.tagged_corpus:
            # Empty textBox and wordEntry
            self.sent_box.delete('0.0', tk.END)
            self.word_entry.delete(0, tk.END)
            # Raise save warnning
            return self.ER.save_warn()
        # Check if any corpus is not imported
        elif not self.cont.corpus:
            # Raise open corpus warnning
            return self.ER.open_warn()
        else:
            # Check if tagged sentence is not empty
            if self.cont.tagged_sent:
                # Append the tagged sentence to the tagged corpus
                self.cont.tagged_corpus.append(self.cont.tagged_sent)
                # Empty the tagged sentence
                self.cont.tagged_sent = []
            # Update the current sentence
            self.cont.current_sent = self.cont.corpus[0]
            # Update the sentence
            self.cont.sentence = self.cont.current_sent
            # Update the corpus
            self.cont.corpus = self.cont.corpus[1:]

    def next_word(self):
        # Check if the cuurent sentence is empty
        if not self.cont.current_sent:
            # Update the current, tagged sentence and the corpus
            self.next_sentence()
        if self.cont.current_sent:
            # Update the word
            self.cont.word = self.cont.current_sent[0]
            # Update the current sentence
            self.cont.current_sent = self.cont.current_sent[1:]

    def tagging(self):
        # Get the tag
        tag = self.tag.get()
        # Check if any tag is selected
        if not tag:
            # Raise tagging error
            return self.ER.tag_error()
        # Tag the word based on the POS style that is selected
        if self.cont.pos_style=='<{}>{}</{}>':
            tagged_word = self.cont.pos_style.format(tag, self.cont.word, tag)
        else:
            tagged_word = self.cont.pos_style.format(self.cont.word, tag)
        # Append the tagged word to the tagged sentence
        self.cont.tagged_sent.append(tagged_word)
        # Go to the next word
        self.next_word()
        # Update textBox and wordEntry
        self.update_textBox()
        self.update_wordEntry()

    def untag_sentence(self):
        # Raise the undoing warnning if the tagged sentence is empty
        if not self.cont.tagged_corpus:
            return self.ER.untagging_error()

        # Fill the tagged sentence
        self.cont.tagged_sent = self.cont.tagged_corpus[-1]

        # Update the tagged corpus
        self.cont.tagged_corpus = self.cont.tagged_corpus[:-1]
        
        # Strip the tagged sentence and assign it to the sentence
        self.cont.sentence = [
            self.cont.tag_striping(i) for i in self.cont.tagged_sent]

    def untag_word(self):
        # check if the tagged sentence is empty and fill it
        if not self.cont.tagged_sent:
            self.untag_sentence()
        if self.cont.tagged_sent:
            # Update the current sentence
            self.cont.current_sent = [self.cont.word] + self.cont.current_sent
            # Update the word
            word = self.cont.tagged_sent[-1]
            self.cont.word = self.cont.tag_striping(word)
            # Update the tagged sentence
            self.cont.tagged_sent = self.cont.tagged_sent[:-1]
            # Update the textbox and wordEntry
            self.update_textBox()
            self.update_wordEntry()

    def merge(self):
        # Check if the current sentence is empty
        if not self.cont.current_sent:

            # Save the sentence as sent
            sent = self.cont.sentence

            # Update the current sentence
            self.next_sentence()

            # Merge last sentence to current sentence
            self.cont.sentence = sent + self.cont.sentence

        # Merge the word by the next word
        self.cont.word += f' {self.cont.current_sent[0]}'

        # Update the current sentence
        self.cont.current_sent = self.cont.current_sent[1:]

        # Update textBox and wordEntry
        self.update_textBox()
        self.update_wordEntry()

    def spliting(self):
        # Check if the word is splitable
        if ' ' not in self.cont.word:
            return self.ER.splitting_error()

        # Split the word
        splited = self.cont.word.split(' ')

        # Append the last word to begining of the current sentence
        self.cont.current_sent = [splited[-1]] + self.cont.current_sent

        # Join the remaining splitted word
        self.cont.word = ' '.join(splited[:-1])
        self.update_wordEntry()

    def retokenize(self):
        # Check if the tagged sentence is empty and raise an error
        if not self.cont.tagged_sent:
            return self.ER.retokenize_error()

        # Append the tagged sentence to the tagged corpus
        self.cont.tagged_corpus.append(self.cont.tagged_sent)

        # Empty the tagged sentence
        self.cont.tagged_sent = []
        
        # Update the sentence and textBox
        self.cont.sentence = self.cont.current_sent
        self.update_textBox()

    def exiting(self):
        # Check if the tagged corpus is empty to quit
        if not self.cont.tagged_corpus:
            self.root.quit()
        else:
            # Raise save warning
            flag = self.ER.exit_wirning()
            # Check if the user presses Yes to save file
            if flag:
                self.cont.save_project()
                self.root.quit()
            # Check if user press No
            elif flag==False:
                self.root.quit()

    def option_window(self):
        # Customize the window 
        self.container = tk.Toplevel()
        self.container.title('Preferences')
        theme = ttk.Style()
        theme.theme_use('clam')
        self.container.geometry('350x185')

        # Create a label, and comboBox to import a tagset
        ttk.Label(
            self.container, 
            text='Select A Tagset:'
            ).grid(
                row=0, 
                column=0, 
                **self.options, 
                sticky='w')

        # Assign a string variable
        self.tagset_name = tk.StringVar()

        ttk.Combobox(
            self.container,
            textvariable=self.tagset_name,
            state='readonly',
            values= list(self.cont.tagset_files.keys())
        ).grid(
            row=0, 
            column=1, 
            columnspan=2, 
            **self.options,
            sticky='e')

        # Set default value to the comboBox
        self.tagset_name.set(self.cont.tagset_name)

        # Add font name label and comboBox
        self.font_name = tk.StringVar()

        ttk.Label(
            self.container, 
            text='Font Name:'
            ).grid(
                row=1, 
                column=0, 
                sticky='w')

        ttk.Combobox(
            self.container, 
            textvariable=self.font_name,
            values=self.cont.font_list,
            state='readonly'
            ).grid(
                row=1, 
                column=1, 
                columnspan=2, 
                **self.options,
                sticky='e')

        # Set a default value
        self.font_name.set(self.cont.font_name)

        # Add font size label and comboBox
        self.font_size = tk.IntVar()

        ttk.Label(
            self.container, 
            text='Font Size: '
            ).grid(
                row=2, 
                column=0, 
                sticky='w')

        ttk.Combobox(
            self.container, 
            textvariable=self.font_size,
            state='readonly',
            values=list(range(8,31,2))
            ).grid(
                row=2, 
                column=1, 
                columnspan=2, 
                **self.options,
                sticky='e')

        # Set a default value to font size
        self.font_size.set(self.cont.font)

        # Add font size label and comboBox
        self.direction = tk.StringVar()
        
        ttk.Label(
            self.container, 
            text='Text Direction: '
            ).grid(
                row=3, 
                column=0, 
                sticky='w')

        ttk.Radiobutton(
            self.container, 
            text='Left to Right', 
            value='left', 
            variable=self.direction
            ).grid(
                row=3, 
                column=1, 
                **self.options)

        ttk.Radiobutton(
            self.container, 
            text='Right to Left', 
            value='right', 
            variable=self.direction
            ).grid(
                row=3, 
                column=2, 
                **self.options)

        # Set a default value to text direction
        self.direction.set(self.cont.dir)

        ttk.Button(
            self.container, 
            text='Cancel', 
            command=self.container.destroy
            ).grid(
                row=4, 
                column=0, 
                **self.options)

        ttk.Button(
            self.container, 
            text='OK', 
            command=self.submiting
            ).grid(
                row=4, 
                column=2, 
                **self.options)

        self.container.grab_set()

    def submiting(self):
        # Update font name, font size, and text direction
        self.cont.tagset_name = self.tagset_name.get()
        self.cont.get_tagset()
        #self.cont.tagset_
        self.cont.font_name = self.font_name.get()
        self.cont.font = self.font_size.get()
        self.cont.dir = self.direction.get()
        # Forget sentence box and word entry
        self.sent_box.grid_forget()
        self.word_entry.grid_forget()
        # Rebuild sentence box and word entry
        self.wordentry()
        self.textbox()
        self.update_categories()
        # Destroy the option window
        self.container.destroy()

    def set_as_default(self):
        self.cont.set_as_default()
        self.tagset_name.set(self.cont.tagset_name)
        self.font_name.set(self.cont.font_name)
        self.font_size.set(self.cont.font)
        self.direction.set(self.cont.dir)


if __name__ == "__main__":
    View()


import os
import tkinter as tk
from tkinter import ttk
from .exeptions import Warnings as WR

class Wizard:
    def __init__(self, root, cont):

        # Make a controller instance
        self.cont = cont

        # Create a wizard window
        root.title('</POSTagger>')
        root.iconphoto(
            False,tk.PhotoImage(
                file=os.path.join(
                    os.path.dirname(os.path.abspath(__file__)), 
                    'imgs','logo.png'
                    )
                )
            )

        root.geometry('680x325')

        # Set a theme to the wizard window
        theme = ttk.Style()
        theme.theme_use('clam')

        # Set padx, pady and sticky for all widgets
        options = {'padx': 5, 'pady': 5}

        # Create import label for project
        ttk.Label(
            root, 
            text='Import a project:'
            ).grid(
                row=0, 
                column=0,
                sticky='w', 
                **options)

        # Create import entry
        self.project_entry = ttk.Entry(root, width=50)
        self.project_entry.grid(
            row=0, 
            column=1,
            columnspan=3,
            sticky='w',
            **options)

        # Create browse button to import a project
        ttk.Button(
            root, 
            text='Import...', 
            command=self.browse_project
            ).grid(
                row=0, 
                column=4,
                sticky='e',
                **options)

        # Create import label for corpus
        ttk.Label(
            root, 
            text='Import a corpus:'
            ).grid(
                row=1, 
                column=0,
                sticky='w',
                **options)

        # Create import entry for corpus
        self.corpus_entry = ttk.Entry(root, width=50)
        self.corpus_entry.grid(
            row=1, 
            column=1,
            columnspan=3,
            sticky='w',
            **options)

        # Create browse button to import corpus
        ttk.Button(
            root, 
            text='Import...', 
            command=self.browse_corpus
            ).grid(
                row=1, 
                column=4,
                sticky='e',
                **options)

        # Create a model label
        ttk.Label(
            root, 
            text='Import Model:'
            ).grid(
                row=2, 
                column=0,
                sticky='w',
                **options)

        # Create model ComboBox
        self.model_name = tk.StringVar()
        self.modelcombo = ttk.Combobox(
            root,
            textvariable=self.model_name,
            values=['This feature is not available yet.'],
            width=49,
            state='disabled')
        self.modelcombo.grid(
            row=2, 
            column=1, 
            columnspan=3,
            sticky='w',
            **options)
        
        # Set a default value for the model entry
        self.model_name.set('This feature is not available yet.')

        # Create a tagset label
        ttk.Label(
            root, 
            text='Slect A Tagset:'
            ).grid(
                row=3, 
                column=0,
                sticky='w',
                **options)
        
        # Create a tagset combobox
        self.tagset_name = tk.StringVar()
        self.tagset_combo = ttk.Combobox(
            root,
            textvariable=self.tagset_name,
            values= list(self.cont.tagset_files.keys()),
            width=49
            )
        self.tagset_combo.grid(
            row=3, 
            column=1, 
            columnspan=3, 
            **options)

        # Set default value to the tagset
        self.tagset_name.set('default')

        # Create a tagging style label
        ttk.Label(
            root, 
            text='POS tagging Style:'
            ).grid(
                row=4, 
                column=0,
                sticky='w',
                **options)
        
        # Create tagging style buttons
        self.style = tk.StringVar()
        self.xml = ttk.Radiobutton(
            root, 
            text='XML', 
            value='<{}>{}</{}>', 
            variable=self.style
            )
        self.xml.grid(
            row=4, 
            column=1,
            sticky='w', 
            **options)

        self.underscore = ttk.Radiobutton(
            root, 
            text='Underscore', 
            value='{}_{}', 
            variable=self.style
            )
        self.underscore.grid(
            row=4, 
            column=2,
            sticky='w',
            **options)

        self.tab = ttk.Radiobutton(
            root, 
            text='Tab', 
            value='{}\t{}\n', 
            variable=self.style
            )
        self.tab.grid(
            row=4, 
            column=3,
            sticky='w',
            **options)

        # Set default value to POS tagging style
        self.style.set('<{}>{}</{}>')
        
        # Create a text direction label
        self.direction = tk.StringVar()
        ttk.Label(
            root, 
            text='Text direction:'
            ).grid(
                row=5, 
                column=0,
                sticky='w',
                **options)

        # Create text direction radioButtons
        ttk.Radiobutton(
            root, 
            text='Left to Right', 
            value='left', 
            variable=self.direction
            ).grid(
                row=5, 
                column=1,
                sticky='w',
                **options)

        ttk.Radiobutton(
            root, 
            text='Right to Left', 
            value='right', 
            variable=self.direction
            ).grid(
                row=5, 
                column=2,
                sticky='w',
                **options)

        # Set default value to text direction
        self.direction.set('left')

        # Create a font size label
        ttk.Label(
            root, 
            text='Font Size:'
            ).grid(
                row=6,
                column=0,
                sticky='w',
                **options)

        # Create a font size comboBox
        self.display_font = tk.IntVar()
        ttk.Combobox(
            root, 
            textvariable=self.display_font, 
            width=5,
            values=list(range(8,31,2))
            ).grid(
                row=6,
                column=1,
                sticky='w',
                **options)

        # Set default value to font size
        self.display_font.set(12)

        # Create OK and cancel buttons
        ttk.Button(
            root, 
            text='Cancel', 
            command=root.destroy
            ).grid(
                row=7, 
                column=0, 
                pady=30)

        ttk.Button(
            root, 
            text='OK', 
            command=lambda: self.starting(root)
            ).grid(
                row=7,
                column=4,
                pady=30)

        root.mainloop()

    def starting(self, root):
        if not self.cont.corpus_path and not self.cont.project_path:
            WR().empty_wizard()
        elif self.cont.corpus_path:
            # Get input variables
            self.cont.tagset_name = self.tagset_name.get()
            self.cont.tagset_path = os.path.join(
                os.path.dirname(os.path.abspath(__file__)),
                'tagsets', 
                self.cont.tagset_files[self.cont.tagset_name])

            self.cont.pos_style = self.style.get()
        self.cont.font = self.display_font.get()
        self.cont.dir = self.direction.get()
        root.destroy()

    def update_widgets(self):
        self.tagset_name.set(self.cont.tagset_name)
        self.style.set(self.cont.pos_style)
        self.display_font.set(self.cont.font)
        self.direction.set(self.cont.dir)

    def browse_project(self):
        # Open the project
        self.cont.open_project()
        if self.cont.project_path:
            # Update widgets
            self.update_widgets()
            # Disable critical features
            self.tagset_combo.config(state='disabled')
            self.xml.config(state='disabled')
            self.underscore.config(state='disabled')
            self.tab.config(state='disabled')
        # Update the project entry
        self.project_entry.delete(0, tk.END)
        self.project_entry.insert(0, self.cont.project_path)

    def browse_corpus(self):
        # Open corpus
        self.cont.open_corpus()
        # Update the corpus entry
        self.corpus_entry.delete(0, tk.END)
        self.corpus_entry.insert(0, self.cont.corpus_path)


if __name__=='__main__':
    app = tk.Tk()
    Wizard(app)



import os
import re
from .Model import Model
import arabic_reshaper
from xml.dom import minidom
from bidi.algorithm import get_display
from .exeptions import Warnings as WR
from tkinter.filedialog import asksaveasfilename, askopenfilename

class Controller:
    def __init__(self):
        self.model = Model()

        # Corpus variables
        self.text = ''
        self.corpus_length = 0
        self.corpus = ''
        self.corpus_path = ''
        self.project_path = ''

        # Working variables
        self.current_sent = ''
        self.sentence = ''
        self.word = ''
        self.tagged_sent = []
        self.tagged_corpus = []
        self.file_name = ''

        # Tagset variables
        self.tagset_files = {
            'default': 'defaultTagset.csv',
            'Brown Tagset': 'BrownTagset.csv',
            'Penn Treebank tagset': 'PENNTagset.csv',
            'Universal POS Tagset': 'UPT.csv',
            'Bijankhan tagset': 'BijankhanTagset.csv'}

        self.tagset_name = 'default'
        self.tagset_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'tagsets', 
            self.tagset_files[self.tagset_name]
            )
        self.tagset = dict()

        self.model_path = ''

        # Style variables
        self.font_list = ['courier', 'Helvetica', 'Tahoma']
        self.pos_style = '<{}>{}</{}>'
        self.font = 12
        self.font_name = 'Tahoma'
        self.dir = 'left'

    def open_corpus(self):
        path = self.get_path()
        if path:
            self.corpus_path = path
            # open the corpus file
            data = self.model.open_file(path)

            # get the file name from the path
            self.file_name = f'project_{os.path.basename(path)}'

            # tokenize the corpus
            self.corpus = self.sent_tokenizer(data)

            # Get normalized text
            self.text = '\n'.join([' '.join(sent) for sent in self.corpus])

            # Get corpus length
            self.corpus_length = len(self.text.split())

            # get the first sentence of the corpus as the current sentence
            self.current_sent = self.corpus[0]
            self.sentence = self.current_sent

            # get the first word of the current sentence as the word
            self.word = self.current_sent[0]
            
            # update the current sentence and the corpus
            self.corpus = self.corpus[1:]
            self.current_sent = self.current_sent[1:]

    def get_tagset(self):
        # Get the path
        self.tagset_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'tagsets', 
            self.tagset_files[self.tagset_name])

        # Open the tagset file
        data = self.model.open_tagset(self.tagset_path)

        # Convert the tagset list to a dictionary
        self.tagset = {key:val for val, key in data}
        return self.tagset
    
    def get_path(
        self, 
        op=True, 
        extension='.txt', 
        filetype=[('Text File','*.txt')]):
        if extension=='.txt' and op:
            initial = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'corpora')
        elif extension=='.txt':
            initial = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'tagsets')
        elif extension=='.proj':
            initial = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'projects')
        elif extension=='.csv':
            initial = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'tagsets')
        elif extension=='.xml':
            initial = os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'outputs')

        if op:
            # Invoke file window to get the opening path
            path = askopenfilename(
                initialdir=initial,
                defaultextension=extension,
                filetypes=filetype
            )
        else:
            path = asksaveasfilename(
                initialdir=initial,
                defaultextension=extension,
                filetypes=filetype
            )
        return path

    def tag_striping(self, string):
        # Create two regex pattern to find 
        # the first one and replace by the second one
        if self.pos_style == '<{}>{}</{}>':
            pat, rep = re.compile(r'(<\w+>)(.+)(<\/\w+>)'), r'\2'
        else:
            pat, rep = re.compile(r'(.+)([_\t]\w+)'), r'\1'
        # Create a striping function to strip tags
        striping = lambda string: pat.sub(rep, string)
        # Strip leading and tailing spaces
        striped = striping(string).strip()
        return striped

    def redirect(self, text):
        reshaped_text = arabic_reshaper.reshape(text)
        redir_text = get_display(reshaped_text)
        return redir_text

    def text_normalizer(self, text):
        # Strip leading and tailing spaces
        text = text.strip()
        # Replace sequence of more than space by one
        text = re.sub(' {2,}', ' ', text)
        # Add space between word and punctuations
        text = re.sub(
            r'(\w)([.:!،؛؟»\]\)\}«\[\(\{])', 
            r'\1 \2', 
            text)
        return text

    def word_normalizer(self, word):
        # Swap Arabic characters and Persians
        translation_src, translation_dst = ';ىكي','یکی؛'
        # Convert latin digits to Persian digits
        translation_src += '0123456789%'
        translation_dst += '۰۱۲۳۴۵۶۷۸۹٪'
        table = word.maketrans(translation_src, translation_dst)
        word = word.translate(table)
        return word.lower()

    def sent_tokenizer(self, text):
        # Normalize the text
        text = self.text_normalizer(text)
        # Tokenize sentences by period, question mark, exclamation mark, and new line
        sents = re.split('(?<=[.؟!\n])\s*', text)
        # Remove empty lines
        sents = [sent for sent in sents if sent!='']
        # Tokenize sentences and words
        normalized_sents = []
        for sent in sents:
            normalized_words = []
            sent = sent.strip().split()
            for word in sent:
                word = self.word_normalizer(word.strip())
                normalized_words.append(word)
            normalized_sents.append(normalized_words)
        return normalized_sents

    def save_project(self):
        # Check if there is somthing to save
        if not self.corpus and not self.tagged_corpus:
            # Raise error if there is nothing to save
            return WR().empty_project()
        path = self.get_path(
            False, 
            '.proj', 
            [('Project Files', '*.proj')])

        if path:
            proj = dict()
            # Corpus variables
            proj['text'] = self.text
            proj['corpus_length'] = self.corpus_length
            proj['corpus'] = self.corpus
            proj['corpus_path'] = self.corpus_path

            # Working variables
            proj['current_sent'] = self.current_sent
            proj['sentence'] = self.sentence
            proj['word'] = self.word
            proj['tagged_sent'] = self.tagged_sent
            proj['tagged_corpus'] = self.tagged_corpus
            proj['file_name'] = self.file_name

            # Tagset variables
            proj['tagset_name'] = self.tagset_name
            proj['tagset_path'] = self.tagset_path
            proj['tagset'] = self.tagset

            proj['model_path'] = self.model_path

            # Style variables
            proj['font_list'] = self.font_list
            proj['pos_style'] = self.pos_style
            proj['font'] = self.font
            proj['font_name'] = self.font_name
            proj['dir'] = self.dir
            self.model.saving(path, proj, 'wb')

    def open_project(self):
        path = self.get_path(
            True, 
            '.proj', 
            [('Project Files', '*.proj')])

        if path:
            self.project_path = path
            proj = self.model.open_file(path, 'rb')

            self.text = proj['text']
            self.corpus_length = proj['corpus_length']
            self.corpus = proj['corpus']
            self.corpus_path = proj['corpus_path']

            # Working variables
            self.current_sent = proj['current_sent']
            self.sentence = proj['sentence']
            self.word = proj['word']
            self.tagged_sent = proj['tagged_sent']
            self.tagged_corpus = proj['tagged_corpus']
            self.file_name = proj['file_name']

            # Tagset variables
            self.tagset_name = proj['tagset_name']
            self.tagset_path = proj['tagset_path']
            self.tagset = proj['tagset']

            self.model_path = proj['model_path']

            # Style variables
            self.font_list = proj['font_list']
            self.pos_style = proj['pos_style']
            self.font = proj['font']
            self.font_name = proj['font_name']
            self.dir = proj['dir']

    def export(self):
        # Export an XML file
        if self.pos_style == '<{}>{}</{}>':
            self.create_xml()
        else:
            # Assign a delimiter based on pos style
            delimiter = ['\n', ''][self.pos_style=='{}\t{}\n']
            # Prepare the tagged text
            text = delimiter.join(' '.join(i) for i in self.tagged_corpus)
            # Get the path
            path = self.get_path(False)
            # Save the file
            self.model.saving(path, text, 'w')

    def create_xml(self):
        flated = [f"<SENT>{''.join(sent)}</SENT>" for sent in self.tagged_corpus]
        xml_string = f"<TEXT>{''.join(flated)}</TEXT>"
        indented =  minidom.parseString(xml_string).toprettyxml()
        path = self.get_path(False, '.xml', [('XML File', '*.xml')])
        if path:
            self.model.saving(path, indented, 'w')
            

if __name__=='__main__':
    Controller()


#!/usr/bin/env python3

import tkinter as tk
from .View import View
from .Wizard import Wizard
from .Controller import Controller

class POSTagger(tk.Tk):
    def __init__(self):
        super().__init__()
        cont = Controller()
        Wizard(self, cont)
        View(cont)

if __name__=='__main__':
    POSTagger()


import os
from setuptools import setup

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="POSTagger",
    version="v0.1.13-beta",
    description="A GUI app for semi-automatic POS tagging",
    long_description=README,
    long_description_content_type="text/markdown",
    keywords = "POS Tagger, Part of Speech Tagger",
    url="https://github.com/asdoost/POSTagger",
    download_url = "https://github.com/asdoost/POSTagger/archive/refs/tags/v0.1.13-beta.tar.gz",
    author="Abbas Safardoost",
    author_email="a.safardoust@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Text Processing :: Linguistic",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3.6"
    ],
    packages=["POSTagger"],
    package_dir={"POSTagger": "POSTagger"},
    package_data={
        "POSTagger": [
            "corpora/*.txt", 
            "imgs/*.png",
            "outputs/*",
            "projects/*.proj",
            "tagsets/*.csv"]
    },
    entry_points={
        "console_scripts": [
            "POSTagger=POSTagger.POSTagger:POSTagger",
        ]
    },
    install_requires=[
        "arabic_reshaper",
        "python-bidi"
        ]
)



import os
from tkinter import font
from tkinter.messagebox import showerror, showwarning, askyesnocancel

class Warnings:
    def __init__(self):
        font1 = font.Font(name='TkCaptionFont', exists=True)
        font1.config(family='courier new', size=11)

    def combine_error(self, lst):
        if len(lst)<2:
            showerror(
                title = "Combine Error", 
                message = "There is no other word in the sentence to combine!"
                )

    def open_corpus_error(self, path):
        if path=='':
            showerror(
                title = "Path Error", 
                message = "The path is empty!")
        elif not os.path.isfile(path):
            showerror(
                title = "File Error", 
                message = "There is no such file!")

    def tag_error(self):
        showerror(
            title = "Tag Error", 
            message = "You didn't select any tag!")
    
    def untagging_error(self):
        showerror(
            title = "Back Error", 
            message = 'You reached the first word of the first sentence. You can\'t go back anymore.')

    def save_warn(self):
        showwarning(
            title='Save Dialog', 
            message='The job is done. Make sure to save it.')

    def open_warn(self):
        showwarning(
            title='Open Dialog', 
            message='You have not added any corpus file yet.')

    def splitting_error(self):
        showerror(
            title = "Splitting Error", 
            message = 'The word can not be splitted anymore.')

    def empty_wizard(self):
        showwarning(
            title='Empty Wizard', 
            message='You didn\'t select a corpus file.')

    def retokenize_error(self):
        showerror(
            title = "Retokenize Error", 
            message = 'There is no tagged sentence to retokenize.')

    def exit_wirning(self):
        return askyesnocancel(
            title= 'Save Warning',
            message='Do you want to save your work?')

    def empty_project(self):
        showerror(
                title = "Empty Project Error", 
                message = "You have not done anything to save!")


import csv
import pickle

class Model:
    def __init__(self):
        pass

    def open_file(self, path, mode='r'):
        # Open the path
        if mode=='rb':
            with open(path, mode) as file:
                data = pickle.load(file)
        else:
            with open(path, mode, encoding='utf8') as file:
                data = file.read()
        return data

    def saving(self, path, text, mode):
        # Save the twxt to selected path
        if mode=='wb':
            with open(path, mode) as file:
                pickle.dump(text, file)
        else:
            with open(path, mode, encoding='utf8') as file:
                file.write(text)

    def open_tagset(self, path):
        # Open CSV tagset file
        file = open(path, 'r', encoding='utf8')
        data = csv.reader(file)
        return data

if __name__=='__main__':
    Model()